Attribute VB_Name = "modFont"
Option Explicit

' NB! I have picked parts of a Owner drawn menu program from G.D. Sever
'   to be able to calculate the size of the text with a specific font. /Henrik
'
' ********************************************************************************
'
'  Project: Menu bitmap module
'  Author:  G. D. Sever    aka "The Hand" (thehand@elitevb.com)
'
' ********************************************************************************

Private Declare Function SystemParametersInfo Lib "user32" Alias "SystemParametersInfoA" (ByVal uAction As Long, ByVal uParam As Long, lpvParam As Any, ByVal fuWinIni As Long) As Long

' Gets the width & height of text in a DC using that DC's currently selected font
' A POINT! YEAH! You know... x and y?
Private Type POINTAPI
    x As Long
    Y As Long
End Type
Private Declare Function GetTextExtentPoint32 Lib "gdi32" Alias "GetTextExtentPoint32A" (ByVal hdc As Long, ByVal lpsz As String, ByVal cbString As Long, lpSize As POINTAPI) As Long
Private gMenuCaps As Collection
Private Declare Function GetDC Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hWnd As Long, ByVal hdc As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
' Logical font type used to size a font with CreateFont
Private Type LOGFONT
    lfHeight As Long
    lfWidth As Long
    lfEscapement As Long
    lfOrientation As Long
    lfWeight As Long
    lfItalic As Byte
    lfUnderline As Byte
    lfStrikeOut As Byte
    lfCharSet As Byte
    lfOutPrecision As Byte
    lfClipPrecision As Byte
    lfQuality As Byte
    lfPitchAndFamily As Byte
    lfFaceName(1 To 32) As Byte
End Type
' Used by SystemParametersInfo to get the non client metrics
Private Const SPI_GETNONCLIENTMETRICS = 41
' Duh! Non client metrics!
Private Type NONCLIENTMETRICS
    cbSize As Long
    iBorderWidth As Long
    iScrollWidth As Long
    iScrollHeight As Long
    iCaptionWidth As Long
    iCaptionHeight As Long
    lfCaptionFont As LOGFONT
    iSMCaptionWidth As Long
    iSMCaptionHeight As Long
    lfSMCaptionFont As LOGFONT
    iMenuWidth As Long
    iMenuHeight As Long
    lfMenuFont As LOGFONT
    lfStatusFont As LOGFONT
    lfMessageFont As LOGFONT
End Type
' Gets display info
Private Declare Function GetDeviceCaps Lib "gdi32" (ByVal hdc As Long, ByVal nIndex As Long) As Long
Const LOGPIXELSY = 90
' Menu font
Public gMenuFont As StdFont
' Individual menu fonts
Public gCustomFonts As Collection
' Creates a new... uh.. font
Private Declare Function CreateFont Lib "gdi32" Alias "CreateFontA" (ByVal h As Long, ByVal W As Long, ByVal E As Long, ByVal O As Long, ByVal W As Long, ByVal i As Long, ByVal u As Long, ByVal S As Long, ByVal C As Long, ByVal OP As Long, ByVal CP As Long, ByVal Q As Long, ByVal PAF As Long, ByVal F As String) As Long


Public Function lGetTextWidth(sText As String, pict As PictureBox) As Long
'aPt
'aPt As POINTAPI
    
    Dim aCap As String
    Dim formDC As Long
    Dim mnuFont     As Long
    Dim aPt2        As POINTAPI
    Dim origFont    As Long
    
    Set gMenuFont = pict.Font
    On Error Resume Next
    If gMenuCaps Is Nothing Then Set gMenuCaps = New Collection
    aCap = gMenuCaps("M" & sText & "-" & frmMain.hWnd)
    aCap = sText
    ' Get the form's DC using its handle
    formDC = GetDC(frmMain.hWnd)
    ' Get the proper font size
    mnuFont = getFontForItem(aCap, formDC)
    ' select the font into the device context
    origFont = SelectObject(formDC, mnuFont)

    ' Get the text width using the form's DC as a reference
    GetTextExtentPoint32 formDC, aCap, Len(aCap), aPt2
    ' Replace the font with the original
    SelectObject formDC, origFont
    ' Release the form's DC back to itself
    ReleaseDC frmMain.hWnd, formDC
    ' Delete the temporary font
    DeleteObject mnuFont
    lGetTextWidth = aPt2.x
    Set gMenuCaps = Nothing
End Function

Private Function getFontForItem(anID As String, hdc As Long, Optional getDefault As Boolean) As Long
   Dim aFont As StdFont
   Dim aNCMETRIC As NONCLIENTMETRICS
   Dim logPixConv As Double

   Dim systemMenuFont As String
   Dim systemMenuSize As Long

   ' Calculate a logical pixels conversion factor
   logPixConv = GetDeviceCaps(hdc, LOGPIXELSY) / 72
   
   On Error Resume Next
   ' Get the nonclient metrics, including system menu height
   aNCMETRIC.cbSize = Len(aNCMETRIC)
   SystemParametersInfo SPI_GETNONCLIENTMETRICS, aNCMETRIC.cbSize, aNCMETRIC, 0
   ' If we're not using custom fonts, or they are not set just use the normal fonts.
       ' use a custom font
       Set aFont = gCustomFonts(anID)
       With aNCMETRIC.lfMenuFont
            If aFont Is Nothing Or getDefault Then
                getFontForItem = CreateFont(-logPixConv * gMenuFont.Size, 0, .lfEscapement, .lfOrientation, gMenuFont.Weight, gMenuFont.Italic, gMenuFont.Underline, gMenuFont.Strikethrough, .lfCharSet, .lfOutPrecision, .lfClipPrecision, .lfQuality, .lfPitchAndFamily, gMenuFont.Name)
            Else
                getFontForItem = CreateFont(-logPixConv * aFont.Size, 0, .lfEscapement, .lfOrientation, aFont.Weight, aFont.Italic, aFont.Underline, aFont.Strikethrough, .lfCharSet, .lfOutPrecision, .lfClipPrecision, .lfQuality, .lfPitchAndFamily, aFont.Name)
            End If
       End With
End Function




