Attribute VB_Name = "modWeeknumber"
Option Explicit

Public Function IsLeapYear(lYear As Long) As Boolean
'Is given year a leap year?
    IsLeapYear = ((lYear Mod 4 = 0) And (lYear Mod 100 <> 0)) Or (lYear Mod 400 = 0)
End Function

Public Function DayOfYear(lDay As Long, lMonth As Long) As Long
' Works out current day of year
Dim vMonth As Variant
    vMonth = Array(0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334)
    DayOfYear = lDay + vMonth(lMonth - 1)
End Function

Public Function Jan1WeekDay(lYear As Long) As Long
' Finds the weekday that Jan 1st of given year was
' Monday=1, Sunday=7
Dim YY As Long
Dim C As Long
Dim G As Long
    YY = (lYear - 1) Mod 100
    C = (lYear - 1) - YY
    G = YY + YY \ 4
    Jan1WeekDay = 1 + (((((C \ 100) Mod 4) * 5) + G) Mod 7)
End Function

Public Function ISOWeekNumber(ByVal lDay As Long, ByVal lMonth As Long, ByVal lYear As Long) As Long
'ISO Week Number
'"The first calendar week of a year is the one that includes the first Thursday
'of that year and the last calendar week of a calendar year is the week
'immediately preceding the first calendar week of the next year", ISO-8601
'Simply: The first week of the year is the earliest week that contains at
'least four days of January. Likewise, the last week of the calendar is the
'last week that contains at least four days of December.
'Microsoft defines the first week of the year as the week that contains
'the 1st of January (which is not always right)
Dim fIsLeapYear As Boolean
Dim fIsYearBeforeLeapYear As Boolean
Dim lDayOfYear As Long
Dim lJan1Weekday As Long
Dim lWeekday As Long
Dim lYearNumber As Long
Dim h As Long
Dim i As Long
Dim j As Long
    ' Find if Year is leapyear, Year-1 is leapyear
    fIsLeapYear = IsLeapYear(lYear)
    fIsYearBeforeLeapYear = IsLeapYear(lYear - 1)
    
    ' Get Day of year, and correct for leap year (if month is later than February)
    lDayOfYear = DayOfYear(lDay, lMonth)
    If fIsLeapYear And lMonth > 2 Then
        lDayOfYear = lDayOfYear + 1
    End If
    ' Find the Weekday for Jan 1st for Year
    lJan1Weekday = Jan1WeekDay(lYear)

    'Find weekday for D M Y
    h = lDayOfYear + (lJan1Weekday - 1)
    lWeekday = 1 + ((h - 1) Mod 7)
    
    ' Find if D M Y falls in Yr-1, WkN 52 or 53
    If lDayOfYear <= (8 - lJan1Weekday) And lJan1Weekday > 4 Then
        lYearNumber = lYear - 1
        If lJan1Weekday = 5 Or (lJan1Weekday = 6 And fIsYearBeforeLeapYear) Then
            ISOWeekNumber = 53
        Else
            ISOWeekNumber = 52
        End If
    Else
        lYearNumber = lYear
    End If
    ' Find if D M Y falls in Yr+1m Wk=1
    If lYearNumber = lYear Then
        If fIsLeapYear Then
            i = 366
        Else
            i = 365
        End If
        If (i - lDayOfYear) < (4 - lWeekday) Then
            lYearNumber = lYear + 1
            ISOWeekNumber = 1
        End If
    End If
    ' Find if D M Y falls in Y, Wk1 to Wk53
    If lYearNumber = lYear Then
        j = lDayOfYear + (7 - lWeekday) + (lJan1Weekday - 1)
        ISOWeekNumber = j \ 7
        If lJan1Weekday > 4 Then
            ISOWeekNumber = ISOWeekNumber - 1
        End If
    End If
End Function
